﻿using System;
using System.ComponentModel;
using tsf_3074.Data;
using tsf_3074.Windows;

namespace tsf_3074.Window
{
	public partial class OverlayWindow : System.Windows.Window
	{
		private readonly HotkeyManager _hotkey;

		public OverlayWindow()
		{
			InitializeComponent();
			DataContext = new OverlayViewModel();
			_hotkey = new HotkeyManager(this);
		}

		private void ReloadAllHotkeys(bool firstTime = false)
		{
			if (!firstTime) _hotkey.UnregisterAllHotkeys();

			TsfFilter.AllFilters.ForEach(f => f.RegisterHotkey(_hotkey));

			// add additional global keybindings (Alt+C)
			_hotkey.RegisterHotkey(VirtualKey.ModAlt, VirtualKey.VkC, () =>
			{
				Console.WriteLine("Configuration hotkey is pressed!");
				_hotkey.UnregisterAllHotkeys(); // unregister all hotkeys to prevent the issue, which the taken hotkeys are not able to be read from the dialog.
				ConfigWindow.ShowConfigDialog();
				Console.WriteLine("Configuration dialog exited.");
				Console.WriteLine("Saving configuration!");
				TsfFilter.AllFilters.ForEach(f =>
				{
					Config.Instance.Hotkeys[f.Name] = new HotkeyPair { vk = f.VK, fs = f.FS };
				});
				Config.SaveConfig();
				Console.WriteLine("Refreshing hotkeys!");
				ReloadAllHotkeys();
			});
		}

		protected override void OnSourceInitialized(EventArgs e)
		{
			base.OnSourceInitialized(e);
			ReloadAllHotkeys(true);
			_hotkey.OnSourceInitialized();
		}

		protected override void OnClosed(EventArgs e)
		{
			_hotkey.OnClosed();
			base.OnClosed(e);
		}
	}

	public class OverlayViewModel : INotifyPropertyChanged
	{
		public event PropertyChangedEventHandler PropertyChanged;

		public string F3074 => Tsf.F3074.GetState();
		public string F3074UL => Tsf.F3074UL.GetState();
		public string F7500 => Tsf.F7500.GetState();
		public string F27K => Tsf.F27K.GetState();
		public string F30K => Tsf.F30K.GetState();
		public string FFullGame => Tsf.FFullGame.GetState();

		public OverlayViewModel()
		{
			Tsf.F3074.OnStateChanged +=
				() => PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(F3074)));
			Tsf.F3074UL.OnStateChanged +=
				() => PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(F3074UL)));
			Tsf.F7500.OnStateChanged +=
				() => PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(F7500)));
			Tsf.F27K.OnStateChanged += () => PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(F27K)));
			Tsf.F30K.OnStateChanged += () => PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(F30K)));
			Tsf.FFullGame.OnStateChanged += () =>
				PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(FFullGame)));
		}
	}
}